/*
 *
 * Outputs flashing depending on pulse variation.
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Copyright (C) 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f509.h>

// oscillator frequency for _delay()
#define _XTAL_FREQ  4000000

#define TEMPO 15
#define TEMPO_LIGHT 20
#define TEMPO_FLASH_ON 18
#define TEMPO_FLASH_OFF 50

 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);
 __IDLOC(0009);

unsigned char cnt, tempo, tempo_light = 0, tempo_flash;
bit neutral, light_on = 0, l1 = 0, l2 = 0, l3 = 0, l4 = 0;

/* ************************************************************************* */
main (void)
{
	OPTION = ~T0CS & ~PSA & ~nGPPU & (nGPWU | T0SE | 0x2);

	TRIS = 0b001001;						// GP0 and GP3 as input, all other as output
	GPIO = 0;								// all ports off
	for (cnt = 0; cnt < 5; cnt++) {			// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
	neutral = 1;							// set neutral indicator
/* ************************************************************************* */
	for (;;) {								// loop for ever
		do {} while (!GP3);					// wait start of pulse
		TMR0 = 255 - 100;					// wait at least 100 * 8 = 800 usec
		do {} while (TMR0 >= 2);			// time elapsed, TMR0 has reached 0 
		do {} while (GP3);					// wait end of pulse
		cnt = TMR0;							// TMR0 is now between 18 and 156, neutral is 88
/* ************************************************************************* */
		if (cnt < 34) {						// 1000 usec
			if (neutral) {
				if (GP0) {
					GP1 = l1 = light_on = 1;
					tempo_light = TEMPO_LIGHT;
				}
				else
					GP1 = l1 = ~l1;
				neutral = 0;
			}
		}
		else if (cnt > 140) {				// 2000 usec
			if (neutral) {
				neutral = 0;
				if (GP0) {
					GP2 = l2 = light_on = 1;
					tempo_light = TEMPO_LIGHT;
				}
				else
					GP2 = l2 = ~l2;
			}
		}
		else if (cnt > 36 && cnt < 80) {	// 1250 usec
			if (neutral && !tempo--) {		// wait some milliseconds before accepting the command
				l3 = ~l3;
				if (l3)
					tempo_flash = 0;
				neutral = 0;				// set neutral state
			}
		}
		else if (cnt > 96 && cnt < 138) {	// 1750 usec
			if (neutral && !tempo--) {		// wait some milliseconds before accepting the command
				GP5 = l4 = ~l4;
				neutral = 0;				// set neutral state
			}
		}
		else if (cnt > 82 && cnt < 94) {	// 1500 usec, neutral
			neutral = 1;
			tempo = TEMPO;
			if (GP0) {
				GP1 = l1 = 0;
				GP2 = l2 = 0;					// reset indicators
				light_on = 0;
			}
		}
		if (!tempo_light) {
			light_on = ~light_on;
			if (l1)
				GP1 = light_on;
			if (l2)
				GP2 = light_on;
			tempo_light = TEMPO_LIGHT;
		}
		else
			tempo_light--;
		if (l3) {
			if (!tempo_flash) {
				GP4 = 1;
				__delay_ms (TEMPO_FLASH_ON);
				GP4 = 0;
				tempo_flash = TEMPO_FLASH_OFF;
			}
			else
				tempo_flash--;
		}
	}
}
